searchData={"items":[{"type":"task","title":"mix nif_call.put_header","doc":"A task responsible for putting the bundled nif_call.h header file into the project.","ref":"Mix.Tasks.NifCall.PutHeader.html"},{"type":"extras","title":"nif_call","doc":"# nif_call\n\n[![Hex.pm](https://img.shields.io/hexpm/v/nif_call.svg?style=flat&color=blue)](https://hex.pm/packages/nif_call)\n\nCall Erlang/Elixir functions from NIF.","ref":"readme.html"},{"type":"extras","title":"Usage - nif_call","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"1. Add `nif_call` as a dependency - nif_call","doc":"Add `nif_call` as a dependency in your `mix.exs` file.\n\n```elixir\ndefp deps do\n  [\n    {:nif_call, \"~> 0.1\"}\n  ]\nend\n```","ref":"readme.html#1-add-nif_call-as-a-dependency"},{"type":"extras","title":"2. Get the header file - nif_call","doc":"It's recommended to use the `nif_call`'s mix task to get the bundled header file. Assuming you're currently in the root directory of your project, run the following command:\n\n```bash\nmix nif_call.put_header\n```\n\nBy default, the header file will be put in the `c_src` directory.  It may look like this:\n\n```bash\n.\n├── Makefile\n├── c_src\n│   ├── demo_nif.cpp\n│   └── nif_call.h          <-- From this repository\n├── lib\n│   └── demo\n│       ├── application.ex\n│       └── demo.ex\n├── mix.exs\n└── mix.lock\n```\n\nYou can also change the directory by passing the `--dir` option.\n\n```bash\nmix nif_call.put_header --dir lib/nif_call\n```\n\nIf there's already a `nif_call.h` file in the target directory, you may want to overwrite it by passing the `--overwrite` option.\n\n```bash\nmix nif_call.put_header --overwrite\n```","ref":"readme.html#2-get-the-header-file"},{"type":"extras","title":"3. Define Evaluator and NIF modules - nif_call","doc":"Define an Evaluator module in your project. The Evaluator module is responsible for evaluating the Erlang/Elixir code. \n\n```elixir\n# lib/demo/evaluator.ex\ndefmodule Demo.Evaluator do\n  use NifCall.Evaluator\nend\n```\n\nTo send the evaluated result back to the caller, `nif_call` needs to inject one NIF function to do that.\n\n```elixir\n# lib/demo/nif.ex\ndefmodule Demo.NIF do\n  use NifCall.NIF\nend\n```\n\nThe default name of this function is `nif_call_evaluated`. You can change it by passing the desired name to the `on_evaluated` option, like this:\n\n```elixir\n# lib/demo/evaluator.ex\ndefmodule Demo.Evaluator do\n  use NifCall.Evaluator, on_evaluated: :my_evaluated\nend\n```\n\n```elixir\n# lib/demo/nif.ex\ndefmodule Demo.NIF do\n  use NifCall.NIF, on_evaluated: :my_evaluated\nend\n```","ref":"readme.html#3-define-evaluator-and-nif-modules"},{"type":"extras","title":"4. Prepare C code - nif_call","doc":"In your NIF code, include `nif_call.h` and define the `NIF_CALL_IMPLEMENTATION` macro before including it.\n\n```c\n// c_src/demo_nif.cpp\n#define NIF_CALL_IMPLEMENTATION\n#include \"nif_call.h\"\n```\n\nAnd remember to initialize nif_call in the `onload` function.\n\n```c\n// c_src/demo_nif.cpp\nstatic int on_load(ErlNifEnv *env, void **, ERL_NIF_TERM) {\n  // initialize nif_call\n  return nif_call_onload(env);\n}\n```\n\nLastly, inject the NIF function:\n\n```c\n// c_src/demo_nif.cpp\nstatic ErlNifFunc nif_functions[] = {\n  // ... your other NIF functions\n\n  // inject nif_call functions\n  // `nif_call_evaluated` is the name of the callback function that will be called by nif_call\n  NIF_CALL_NIF_FUNC(nif_call_evaluated),\n\n  // of course, you can change the name of the callback function\n  // but remember to change it in the Elixir code as well\n  // NIF_CALL_NIF_FUNC(my_evaluated),\n};\n```","ref":"readme.html#4-prepare-c-code"},{"type":"extras","title":"5. Call Erlang/Elixir functions from NIF - nif_call","doc":"Let's try to implement a simple function that adds 1 to the given value and sends the intermediate result to Elixir for further processing. The result of the Elixir callback function is returned as the final result.\n\nFirst, we need to start the Evaluator process in the Elixir code.\n\n```elixir\n# lib/demo/application.ex\ndefmodule Demo.Application do\n  @moduledoc false\n\n  use Application\n\n  @impl true\n  def start(_type, _args) do\n    children = [\n      {Demo.Evaluator, [nif_module: Demo.NIF, process_options: [name: Demo.Evaluator]]}\n    ]\n\n    opts = [strategy: :one_for_one, name: Demo.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```\n\nSecond, implement the `add_one` function in the Elixir code.\n\n```elixir\n# lib/demo/demo.ex\ndefmodule Demo do\n  @doc \"\"\"\n  Add 1 to the `value` in NIF and send the intermediate result to\n  Elixir for further processing using the `callback` function.\n\n  The result of the `callback` function is returned as the final result.","ref":"readme.html#5-call-erlang-elixir-functions-from-nif"},{"type":"extras","title":"Examples - nif_call","doc":"iex> Demo.add_one(1, fn result -> result * 2 end)\n      4\n\n  \"\"\"\n  def add_one(value, callback) do\n    # remember to change the name of the Evaluator module if you have changed it\n    # and pass both the evaluator and the callback function to the NIF\n    evaluator = Process.whereis(Demo.Evaluator)\n    Demo.NIF.add_one(value, evaluator, callback)\n  end\nend\n```\n\nAfter that, implement the `add_one` function in the NIF C code.\n\n```c\n// c_src/demo_nif.cpp\nstatic ERL_NIF_TERM add_one(ErlNifEnv* env, int argc, const ERL_NIF_TERM argv[]) {\n  ErlNifSInt64 a;\n  ErlNifPid evaluator;\n  ERL_NIF_TERM fun = argv[2];\n\n  if (!enif_get_int64(env, argv[0], &a) || !enif_get_local_pid(env, argv[1], &evaluator)) return enif_make_badarg(env);\n  ERL_NIF_TERM result_term = enif_make_int64(env, a + 1);\n\n  // send the intermediate result to Elixir for further processing\n  // `make_nif_call` will return the result of the callback function\n  // which is the final result in this case\n  return make_nif_call(env, evaluator, fun, result_term);\n}\n```\n\nMost importantly, don't forget to add the NIF function to the `nif_functions` array, and **they have to be marked as dirty NIF functions**.\n\n```c\n// c_src/demo_nif.cpp\nstatic ErlNifFunc nif_functions[] = {\n  // ... your other NIF functions\n\n  // inject nif_call functions\n  NIF_CALL_NIF_FUNC(nif_call_evaluated),\n\n  // add the NIF function\n  // NIF functions that calls Elixir functions have to be marked as dirty\n  // either ERL_NIF_DIRTY_JOB_CPU_BOUND or ERL_NIF_DIRTY_JOB_IO_BOUND\n  {\"add_one\", 3, add_one, ERL_NIF_DIRTY_JOB_CPU_BOUND},\n};\n```\n\nNow, you can call the `add_one` function from Elixir.\n\n```elixir\niex> Demo.add_one(1, fn result -> result * 2 end)\n4\n```\n\nCongratulations! You have successfully called an Elixir function from NIF.\n\nThere's a slightly more complex example in the `example` directory, which shows that you can make multiple calls to Elixir functions from NIF and \nuse the intermediate results in the next call.","ref":"readme.html#examples"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}